import React from 'react';
import { createStyledComponent } from '@cloudflare/style-container';
import { Div } from '@cloudflare/elements';
import { SELECTABLE_ID, EXPANDABLE_ID } from './Table';
import { Icon } from '@cloudflare/component-icon';

// A11Y-94: Button to trigger sorting for keyboard users.
const SortButton = createStyledComponent(
  () => ({
    background: 'none',
    color: 'inherit',
    alignItems: 'center',
    display: 'flex',
    border: 'none',
    padding: 0,
    font: 'inherit',
    cursor: 'pointer',
    outline: 'inherit',
    width: '100%',
    '&:focus': {
      outline: 'solid',
      outlineOffset: 3,
      outlineStyle: 'auto',
      outlineColor: 'gray.8'
    }
  }),
  'button'
);

type HeaderCellProps = {
  isLoading?: boolean;
  hasError?: boolean;
  isEmpty?: boolean;
  column: any;
  handleSortByClick?: () => void;
  selectAllKey?: string;
};

const HeaderCellWrapper = createStyledComponent<'th', HeaderCellProps>(
  ({ theme, isLoading, hasError, isEmpty, column }) => ({
    display: 'flex',
    alignItems: 'center',
    fontSize: 2,
    fontWeight: 600,
    color: 'gray.1',
    pl: 2,
    pr: 2,
    pb: 1,
    pt: 1,
    backgroundColor: theme.colors.background,
    borderBottom: `1px solid ${theme.colors.gray[7]}`,
    transition: 'opacity 0.2s, background-color 0.2s',
    ...(column.canSort ? { cursor: 'pointer' } : {}),
    '&:hover': {
      backgroundColor:
        column.canSort &&
        !isLoading &&
        !hasError &&
        !isEmpty &&
        theme.colors.gray[9]
    }
  }),
  'th'
);

const HeaderCell = ({
  column,
  isLoading,
  isEmpty,
  hasError,
  selectAllKey
}: HeaderCellProps) => (
  <HeaderCellWrapper
    {...column.getHeaderProps({
      style: {
        ...((column.id === SELECTABLE_ID || column.id === EXPANDABLE_ID) && {
          flex: '0 0 auto'
        })
      }
    })}
    aria-sort={
      column.isSorted
        ? column.isSortedDesc
          ? 'descending'
          : 'ascending'
        : 'none'
    }
    isLoading={isLoading}
    hasError={hasError}
    isEmpty={isEmpty}
    column={column}
  >
    {column.canSort ? (
      <SortButton onClick={() => column.toggleSortBy()}>
        {column.render('Header', { selectAllKey })}
        {column.canSort && !isLoading && !hasError && !isEmpty && (
          <Div
            display="flex"
            ml={2}
            style={{ transform: column.isSortedDesc ? 'rotate(-180deg)' : '' }}
          >
            {column.isSorted ? (
              <Icon type="caret-up" color="gray.0" ariaLabel="" />
            ) : (
              <Icon type="reorder" color="gray.0" ariaLabel="" size={12} />
            )}
          </Div>
        )}
      </SortButton>
    ) : (
      column.render('Header', { selectAllKey })
    )}
  </HeaderCellWrapper>
);

export default HeaderCell;
